package com.xhkjedu.sexam.service.exam;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.xhkjedu.sexam.mapper.exam.EBaseMapper;
import com.xhkjedu.sexam.mapper.exam.EClassMapper;
import com.xhkjedu.sexam.mapper.exam.ESubjectInvigilateMapper;
import com.xhkjedu.sexam.mapper.exam.ESubjectMapper;
import com.xhkjedu.sexam.model.exam.ESubjectInvigilate;
import com.xhkjedu.sexam.vo.exam.EITeacherVo;
import com.xhkjedu.sexam.vo.paper.EPTeacherVo;
import com.xhkjedu.utils.N_Utils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class ESubjectInvigilateService {
    @Resource
    private ESubjectInvigilateMapper eSubjectInvigilateMapper;
    @Resource
    private EBaseMapper eBaseMapper;
    @Resource
    private ESubjectMapper eSubjectMapper;
    @Resource
    private EClassMapper eClassMapper;

    //设置监考人--设置
    @Transactional(rollbackFor = Exception.class)
    public void setTeacher(Integer esid, String subjectid, Integer createid, List<ESubjectInvigilate> teachers) {
        eSubjectInvigilateMapper.delByEsId(esid);//删除考试科目监考教师
        int timestamp = N_Utils.getSecondTimestamp();
        teachers.forEach(t -> {
            t.setEsid(esid);
            t.setSubjectid(subjectid);
            t.setCreateid(createid);
            t.setCreatetime(timestamp);
        });
        eSubjectInvigilateMapper.insertList(teachers);
    }

    //监考设置列表
    public Map listSubject(Integer examid) {
        Map map = eBaseMapper.findById(examid);
        List<Map> subjects = eSubjectInvigilateMapper.listSubject(examid);//监考设置列表
        String esids = subjects.stream().map(s -> s.get("esid").toString()).collect(Collectors.joining(","));
        List<EITeacherVo> users = eSubjectInvigilateMapper.listUserByEsIds(esids);
        List<Map> classes = eClassMapper.listByExamId(examid);//考试班级
        int classnum = classes.size();
        for (Map s : subjects) {
            Integer esid = (Integer) s.get("esid");
            if (N_Utils.isListEmpty(users)) users = new ArrayList<>();
            String majorname = users.stream().filter(u -> esid.equals(u.getEsid()) && u.getMajored() == 1)
                    .map(u -> u.getUsername()).distinct().collect(Collectors.joining("、"));
            s.put("majorname", majorname);
            long einum;
            if (N_Utils.isNotEmpty(majorname)) {
                einum = 0;
            } else {
                einum = users.stream().filter(u -> esid.equals(u.getEsid()) && !N_Utils.isEmptyInteger(u.getClassid()))
                        .map(u -> u.getClassid()).distinct().count();
                einum = classnum - einum;
            }
            s.put("einum", einum);
            for (Map c : classes) {
                Integer classid = (Integer) c.get("classid");
                List<Map> teachers=new ArrayList<>();
                for (EITeacherVo u : users) {
                    if(esid.equals(u.getEsid())&& (1 == u.getMajored() || classid.equals(u.getClassid()))){
                        Map t=new HashMap();
                        t.put("teacherid",u.getTeacherid());
                        t.put("username",u.getUsername());
                        t.put("loginname",u.getLoginname());
                        teachers.add(t);
                    }
                }
                c.put("teachers", teachers.stream().distinct().collect(Collectors.toList()));
            }
            //关闭引用检测，避免循环给classes赋值出现"$ref":"$[0]"现象
            String str = JSON.toJSONString(classes, SerializerFeature.DisableCircularReferenceDetect);
            s.put("classes", JSON.parseArray(str, Map.class));
        }
        map.put("subjects", subjects);
        return map;
    }

    //监考人查看
    public Map listByEsId(Integer esid) {
        Map map = new HashMap();
        Integer examId = eSubjectMapper.getExamIdByEsId(esid);//根据考试科目id获取考试id
        List<Map> list = eClassMapper.listByExamId(examId);//考试班级
        List<Map> users = eSubjectInvigilateMapper.listByEsId(esid);//考试监考人
        map.put("classes", list);
        map.put("teachers", users);
        return map;
    }

    //监考人删除
    public void delByEsId(Integer esid) {
        eSubjectInvigilateMapper.delByEsId(esid);//删除考试科目监考教师
    }

    //教师监考-监控列表
    public List<Map> listJkSubject(Integer examid, Integer teacherid, Integer esstate) {
        int timestamp = N_Utils.getSecondTimestamp();
        String strtime = N_Utils.getStrtimeTimestamp(timestamp, "yyyy-MM-dd HH:mm:ss");
        List<Map> list = eSubjectInvigilateMapper.listJkSubject(examid, teacherid, esstate, strtime);
        for (Map map : list) {
            List<Map> classes = JSON.parseArray(map.get("classnames").toString(), Map.class);
            List<Map> classList = new ArrayList<>();
            Integer majored = (Integer) map.get("majored");
            if (majored == 1) {//主监考
                classList.addAll(classes);
            } else {
                String[] classids2 = map.get("classids").toString().split(",");
                for (String classid : classids2) {
                    Map cmap = classes.stream().filter(c -> c.get("classid").toString().equals(classid)).findFirst().orElse(null);
                    if (cmap != null) classList.add(cmap);
                }
            }
            map.remove("classids");
            map.remove("classnames");
            map.put("classList", classList);
        }
        return list;
    }

    //根据考试科目id获取考试科目基本信息
    public Map getESubjectByEsId(Integer esid) {
        return eSubjectMapper.getESubjectByEsId(esid);
    }

    //缺考学生列表
    public List<Map> listMissStu(EPTeacherVo vo) {
        return eSubjectInvigilateMapper.listMissStu(vo);
    }
}
